@@ -66,7 +66,7 @@ if __name__ == "__main__":
             print("  LIBXSMM_MESSAGE(\"================================================================================\")")
             if (0 != prefetch):
                 print("  LIBXSMM_UNUSED(pa); LIBXSMM_UNUSED(pb); LIBXSMM_UNUSED(pc);")
-            print("  LIBXSMM_INLINE_XGEMM(float, int/*libxsmm_blasint not req.*/, LIBXSMM_FSYMBOL(sgemm),\n" \
+            print("  LIBXSMM_INLINE_XGEMM(float, int/*libxsmm_blasint not req.*/,\n" \
                   "LIBXSMM_FLAGS, LIBXSMM_LD(" + str(m) + ", " + str(n) + "), LIBXSMM_LD(" + str(n) + ", " + str(m) + "), " + str(k) + ",\n" \
                   "LIBXSMM_ALPHA, LIBXSMM_LD(a, b), LIBXSMM_LD(" + str(m) + ", " + str(n) + "), LIBXSMM_LD(b, a), " + str(k) + ",\n" \
                   "LIBXSMM_BETA, c, LIBXSMM_LD(" + str(m) + ", " + str(n) + "));")
@@ -94,7 +94,7 @@ if __name__ == "__main__":
             print("  LIBXSMM_MESSAGE(\"No specific instruction set extension found for specialization!\")*/")
             if (0 != prefetch):
                 print("  LIBXSMM_UNUSED(pa); LIBXSMM_UNUSED(pb); LIBXSMM_UNUSED(pc);")
-            print("  LIBXSMM_INLINE_XGEMM(double, int/*libxsmm_blasint not req.*/, LIBXSMM_FSYMBOL(dgemm),\n" \
+            print("  LIBXSMM_INLINE_XGEMM(double, int/*libxsmm_blasint not req.*/,\n" \
                   "LIBXSMM_FLAGS, LIBXSMM_LD(" + str(m) + ", " + str(n) + "), LIBXSMM_LD(" + str(n) + ", " + str(m) + "), " + str(k) + ",\n" \
                   "LIBXSMM_ALPHA, LIBXSMM_LD(a, b), LIBXSMM_LD(" + str(m) + ", " + str(n) + "), LIBXSMM_LD(b, a), " + str(k) + ",\n" \
                   "LIBXSMM_BETA, c, LIBXSMM_LD(" + str(m) + ", " + str(n) + "));")
