@@ -173,8 +173,10 @@ void libxsmm_generator_gemm_init_micro_kernel_config_fullvector( libxsmm_micro_k
               (strcmp( i_arch, "skx" ) == 0)    ) {
     if ((strcmp( i_arch, "knc" ) == 0)) {
       io_micro_kernel_config->instruction_set = LIBXSMM_X86_IMCI;
+    } else if ((strcmp( i_arch, "knl" ) == 0)) {
+      io_micro_kernel_config->instruction_set = LIBXSMM_X86_AVX512_MIC;
     } else {
-      io_micro_kernel_config->instruction_set = LIBXSMM_X86_AVX512;
+      io_micro_kernel_config->instruction_set = LIBXSMM_X86_AVX512_CORE;
     }
     io_micro_kernel_config->vector_reg_count = 32;
     io_micro_kernel_config->use_masking_a_c = i_use_masking_a_c;
@@ -433,7 +435,7 @@ void libxsmm_generator_gemm_add_flop_counter( libxsmm_generated_code*         io
                                                const libxsmm_gemm_descriptor* i_xgemm_desc ) {
   if ( io_generated_code->code_type == 0 ) {
     char l_new_code[512];
-    int l_max_code_length = 511;
+    const unsigned int l_max_code_length = sizeof(l_new_code) - 1;
     int l_code_length = 0;
 
     l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifndef NDEBUG\n" );
@@ -444,7 +446,7 @@ void libxsmm_generator_gemm_add_flop_counter( libxsmm_generated_code*         io
     libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n" );
     libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    l_code_length = LIBXSMM_SNPRINTF( l_new_code, 511, "libxsmm_num_total_flops += %u;\n", 2 * i_xgemm_desc->m * i_xgemm_desc->n * i_xgemm_desc->k);
+    l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "libxsmm_num_total_flops += %u;\n", 2u * i_xgemm_desc->m * i_xgemm_desc->n * i_xgemm_desc->k);
     libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n" );
     libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
@@ -592,8 +594,9 @@ void libxsmm_generator_gemm_load_C( libxsmm_generated_code*             io_gener
       libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
       return;
     }
-  } else if (i_micro_kernel_config->instruction_set == LIBXSMM_X86_IMCI    ||
-             i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512     ) {
+  } else if (i_micro_kernel_config->instruction_set == LIBXSMM_X86_IMCI        ||
+             i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_MIC  ||
+             i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE    ) {
     if ( (i_n_blocking > 30) || (i_n_blocking < 1) || (i_m_blocking != i_micro_kernel_config->vector_length) ) {
       libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
       return;
@@ -661,8 +664,9 @@ void libxsmm_generator_gemm_store_C( libxsmm_generated_code*             io_gene
       libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
       return;
     }
-  } else if (i_micro_kernel_config->instruction_set == LIBXSMM_X86_IMCI    ||
-             i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512     ) {
+  } else if (i_micro_kernel_config->instruction_set == LIBXSMM_X86_IMCI        ||
+             i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_MIC  ||
+             i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE    ) {
     if ( (i_n_blocking > 30) || (i_n_blocking < 1) || (i_m_blocking != i_micro_kernel_config->vector_length) ) {
       libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
       return;
