@@ -69,7 +69,8 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_initialize_mask( l
                                    LIBXSMM_X86_INSTR_KMOV,
                                    i_gp_reg_mapping->gp_reg_help_5,
                                    LIBXSMM_X86_IMCI_AVX512_MASK );
-  } else if ( i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512 ) {
+  } else if ( i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_MIC ||
+              i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE   ) {
     libxsmm_x86_instruction_mask_move( io_generated_code,
                                    LIBXSMM_X86_INSTR_KMOVW,
                                    i_gp_reg_mapping->gp_reg_help_5,
