@@ -31,8 +31,8 @@
 #include "libxsmm_intrinsics_x86.h"
 #include "libxsmm_cpuid_x86.h"
 #include "libxsmm_gemm_diff.h"
+#include "libxsmm_gemm_ext.h"
 #include "libxsmm_hash.h"
-#include "libxsmm_gemm.h"
 
 #if defined(__TRACE)
 # include "libxsmm_trace.h"
@@ -45,6 +45,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <math.h>
 #if !defined(NDEBUG)
 # include <assert.h>
 # include <errno.h>
@@ -60,6 +61,18 @@
 # include <unistd.h>
 # include <fcntl.h>
 #endif
+#if defined(LIBXSMM_VTUNE)
+# include <jitprofiling.h>
+# define LIBXSMM_VTUNE_JITVERSION 2
+# if (2 == LIBXSMM_VTUNE_JITVERSION)
+#   define LIBXSMM_VTUNE_JIT_DESC_TYPE iJIT_Method_Load_V2
+#   define LIBXSMM_VTUNE_JIT_LOAD iJVM_EVENT_TYPE_METHOD_LOAD_FINISHED_V2
+# else
+#   define LIBXSMM_VTUNE_JIT_DESC_TYPE iJIT_Method_Load
+#   define LIBXSMM_VTUNE_JIT_LOAD iJVM_EVENT_TYPE_METHOD_LOAD_FINISHED
+# endif
+# define LIBXSMM_VTUNE_JIT_UNLOAD iJVM_EVENT_TYPE_METHOD_UNLOAD_START
+#endif
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
 #endif
@@ -118,8 +131,16 @@
 /* flag fused into the memory address of a code version in case of collision */
 #define LIBXSMM_HASH_COLLISION (1ULL << (8 * sizeof(void*) - 1))
 
+#if 16 >= (LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+# define LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE 16
+#elif 32 >= (LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+# define LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE 32
+#else
+# define LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE LIBXSMM_GEMM_DESCRIPTOR_SIZE
+#endif
+
 typedef union LIBXSMM_RETARGETABLE internal_regkey {
-  char simd[32];
+  char simd[LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE];
   libxsmm_gemm_descriptor descriptor;
 } internal_regkey;
 
@@ -131,9 +152,25 @@ typedef struct LIBXSMM_RETARGETABLE internal_regentry {
   } function;
   /* statically generated code (=0), dynamically generated code (>0). */
   unsigned int size;
+#if defined(LIBXSMM_VTUNE)
+  unsigned int id;
+#endif
 } internal_regentry;
 
-LIBXSMM_DEBUG(LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL unsigned int internal_ncollisions = 0;)
+LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL struct LIBXSMM_RETARGETABLE {
+  unsigned int ntry, ncol, njit, nsta;
+} internal_statistic[2/*DP/SP*/][3/*sml/med/big*/];
+
+LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL unsigned int internal_statistic_sml = 13;
+LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL unsigned int internal_statistic_med = 23;
+LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL unsigned int internal_statistic_mnk = LIBXSMM_MAX_M;
+
+#if defined(NDEBUG)
+LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_verbose = 0; /* quiet */
+#else
+LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_verbose = 1; /* verbose */
+#endif
+
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL internal_regkey* internal_registry_keys = 0;
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL internal_regentry* internal_registry = 0;
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL unsigned int internal_teardown = 0;
@@ -151,8 +188,7 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL unsigned int internal_teardown
 #endif
 
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_prefetch = LIBXSMM_MAX(INTERNAL_PREFETCH, 0);
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_target_arch = LIBXSMM_TARGET_ARCH_GENERIC;
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL const char* internal_target_archid = 0;
+LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_target_archid = LIBXSMM_TARGET_ARCH_GENERIC;
 
 #if !defined(LIBXSMM_OPENMP)
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_reglock[] = {
@@ -164,9 +200,11 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
 #endif
 
 #if defined(__GNUC__)
+# define LIBXSMM_INIT
   /* libxsmm_init already executed via GCC constructor attribute */
 # define INTERNAL_FIND_CODE_INIT(VARIABLE) assert(0 != (VARIABLE))
 #else /* lazy initialization */
+# define LIBXSMM_INIT libxsmm_init();
   /* use return value of internal_init to refresh local representation */
 # define INTERNAL_FIND_CODE_INIT(VARIABLE) if (0 == (VARIABLE)) VARIABLE = internal_init()
 #endif
@@ -208,14 +246,14 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
 
 #if defined(LIBXSMM_CACHESIZE) && (0 < (LIBXSMM_CACHESIZE))
 # define INTERNAL_FIND_CODE_CACHE_DECL(CACHE_ID, CACHE_KEYS, CACHE, CACHE_HIT) \
-  static LIBXSMM_TLS union { libxsmm_gemm_descriptor desc; char padding[32]; } CACHE_KEYS[LIBXSMM_CACHESIZE]; \
+  static LIBXSMM_TLS union { libxsmm_gemm_descriptor desc; char padding[LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE]; } CACHE_KEYS[LIBXSMM_CACHESIZE]; \
   static LIBXSMM_TLS libxsmm_xmmfunction CACHE[LIBXSMM_CACHESIZE]; \
   static LIBXSMM_TLS unsigned int CACHE_ID = (unsigned int)(-1); \
   static LIBXSMM_TLS unsigned int CACHE_HIT = LIBXSMM_CACHESIZE
 # define INTERNAL_FIND_CODE_CACHE_BEGIN(CACHE_ID, CACHE_KEYS, CACHE, CACHE_HIT, RESULT, DESCRIPTOR) \
-  assert(32 >= LIBXSMM_GEMM_DESCRIPTOR_SIZE); \
+  assert(LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE >= LIBXSMM_GEMM_DESCRIPTOR_SIZE); \
   /* search small cache starting with the last hit on record */ \
-  i = libxsmm_gemm_diffn(DESCRIPTOR, &(CACHE_KEYS)->desc, CACHE_HIT, LIBXSMM_CACHESIZE, 32); \
+  i = libxsmm_gemm_diffn(DESCRIPTOR, &(CACHE_KEYS)->desc, CACHE_HIT, LIBXSMM_CACHESIZE, LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE); \
   if ((LIBXSMM_CACHESIZE) > i && (CACHE_ID) == internal_teardown) { /* cache hit, and valid */ \
     (RESULT).function.xmm = (CACHE)[i]; \
     CACHE_HIT = i; \
@@ -252,7 +290,7 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
 #if (0 != LIBXSMM_JIT)
 # define INTERNAL_FIND_CODE_JIT(DESCRIPTOR, CODE, RESULT) \
   /* check if code generation or fix-up is needed, also check whether JIT is supported (CPUID) */ \
-  if (0 == (RESULT).function.pmm && LIBXSMM_X86_AVX <= internal_target_arch) { \
+  if (0 == (RESULT).function.pmm /* code version does not exist */ && LIBXSMM_X86_AVX <= internal_target_archid) { \
     /* instead of blocking others, a try-lock would allow to let others to fallback to BLAS (return 0) during lock-time */ \
     INTERNAL_FIND_CODE_LOCK(lock, i); /* lock the registry entry */ \
     /* re-read registry entry after acquiring the lock */ \
@@ -264,6 +302,7 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
       if (0 == diff) { \
         /* found a conflict-free registry-slot, and attempt to build the kernel */ \
         internal_build(DESCRIPTOR, &(RESULT)); \
+        internal_update_statistic(DESCRIPTOR, 1, 0); \
         if (0 != (RESULT).function.pmm) { /* synchronize registry entry */ \
           internal_registry_keys[i].descriptor = *(DESCRIPTOR); \
           *(CODE) = RESULT; \
@@ -278,7 +317,7 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
           const unsigned int index = LIBXSMM_HASH_MOD(LIBXSMM_HASH_VALUE(hash), LIBXSMM_REGSIZE); \
           i = (index != i ? index : LIBXSMM_HASH_MOD(index + 1, LIBXSMM_REGSIZE)); \
           i0 = i; /* keep starting point of free-slot-search in mind */ \
-          LIBXSMM_DEBUG(++internal_ncollisions;) \
+          internal_update_statistic(DESCRIPTOR, 0, 1); \
           INTERNAL_FIND_CODE_WRITE(CODE, collision); /* fix-up existing entry */ \
           diff0 = diff; /* no more fix-up */ \
         } \
@@ -314,7 +353,7 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
     (CODE) += i; /* actual entry */ \
     do { \
       INTERNAL_FIND_CODE_READ(CODE, flux_entry.function.pmm); /* read registered code */ \
-      if (0 != flux_entry.function.pmm) { \
+      if (0 != flux_entry.function.pmm) { /* code version exists */ \
         if (0 == diff0) { \
           if (0 == (LIBXSMM_HASH_COLLISION & flux_entry.function.imm)) { /* check for no collision */ \
             /* calculate bitwise difference (deep check) */ \
@@ -406,13 +445,133 @@ return flux_entry.function.xmm
   M, N, K, PLDA, PLDB, PLDC, PALPHA, PBETA, PREFETCH, dmm)
 
 
-LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_register_static_code(
-  const libxsmm_gemm_descriptor* desc, unsigned int index, unsigned int hash, libxsmm_xmmfunction src,
-  internal_regentry* registry, unsigned int* registered, unsigned int* total)
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_update_statistic(const libxsmm_gemm_descriptor* desc,
+  unsigned ntry, unsigned ncol)
+{
+  assert(0 != desc);
+  {
+    const unsigned long long size = LIBXSMM_MNK_SIZE(desc->m, desc->n, desc->k);
+    const int precision = (0 == (LIBXSMM_GEMM_FLAG_F32PREC & desc->flags) ? 0 : 1);
+    int bucket = 2/*big*/;
+
+    if (LIBXSMM_MNK_SIZE(internal_statistic_sml, internal_statistic_sml, internal_statistic_sml) >= size) {
+      bucket = 0;
+    }
+    else if (LIBXSMM_MNK_SIZE(internal_statistic_med, internal_statistic_med, internal_statistic_med) >= size) {
+      bucket = 1;
+    }
+
+    /* TODO: use atomic updates */
+    internal_statistic[precision][bucket].ncol += ncol;
+    internal_statistic[precision][bucket].ntry += ntry;
+  }
+}
+
+
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE const char* internal_get_target_arch(int id);
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE const char* internal_get_target_arch(int id)
+{
+  const char* target_arch = 0;
+  switch (id) {
+    case LIBXSMM_X86_AVX512_CORE: {
+      target_arch = "skx";
+    } break;
+    case LIBXSMM_X86_AVX512_MIC: {
+      target_arch = "knl";
+    } break;
+    case LIBXSMM_X86_AVX2: {
+      target_arch = "hsw";
+    } break;
+    case LIBXSMM_X86_AVX: {
+      target_arch = "snb";
+    } break;
+    case LIBXSMM_X86_SSE4_2: {
+      target_arch = "wsm";
+    } break;
+    case LIBXSMM_X86_SSE4_1: {
+      target_arch = "sse4";
+    } break;
+    case LIBXSMM_X86_SSE3: {
+      target_arch = "sse3";
+    } break;
+    case LIBXSMM_TARGET_ARCH_GENERIC: {
+      target_arch = "generic";
+    } break;
+    default: if (LIBXSMM_X86_GENERIC <= id) {
+      target_arch = "x86";
+    }
+    else {
+      target_arch = "unknown";
+    }
+  }
+
+  assert(0 != target_arch);
+  return target_arch;
+}
+
+
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE unsigned int internal_print_statistic(FILE* ostream, const char* target_arch, int precision, unsigned int linebreaks, unsigned int indent)
+{
+  int printed = 0;
+  assert(0 != ostream && 0 != target_arch && (0 <= precision && precision < 2));
+
+  if (/* omit to print anything if it is superfluous */
+    0 != internal_statistic[precision][0/*SML*/].ntry ||
+    0 != internal_statistic[precision][0/*SML*/].njit ||
+    0 != internal_statistic[precision][0/*SML*/].nsta ||
+    0 != internal_statistic[precision][0/*SML*/].ncol ||
+    0 != internal_statistic[precision][1/*MED*/].ntry ||
+    0 != internal_statistic[precision][1/*MED*/].njit ||
+    0 != internal_statistic[precision][1/*MED*/].nsta ||
+    0 != internal_statistic[precision][1/*MED*/].ncol ||
+    0 != internal_statistic[precision][2/*BIG*/].ntry ||
+    0 != internal_statistic[precision][2/*BIG*/].njit ||
+    0 != internal_statistic[precision][2/*BIG*/].nsta ||
+    0 != internal_statistic[precision][2/*BIG*/].ncol)
+  {
+    char title[256], sml[256], med[256], big[256];
+
+    LIBXSMM_SNPRINTF(sml, sizeof(sml), "%u..%u",                     0u, internal_statistic_sml);
+    LIBXSMM_SNPRINTF(med, sizeof(sml), "%u..%u", internal_statistic_sml, internal_statistic_med);
+    LIBXSMM_SNPRINTF(big, sizeof(sml), "%u..%u", internal_statistic_med, internal_statistic_mnk);
+    {
+      unsigned int n = 0;
+      for (n = 0; 0 != target_arch[n] && n < sizeof(title); ++n) { /* toupper */
+        const char c = target_arch[n];
+        title[n] = (char)(('a' <= c && c <= 'z') ? (c - 32) : c);
+      }
+      LIBXSMM_SNPRINTF(title + n, sizeof(title) - n, "/%s", 0 == precision ? "DP" : "SP");
+      for (n = 0; n < linebreaks; ++n) fprintf(ostream, "\n");
+    }
+    fprintf(ostream, "%*s%-10s %6s %6s %6s %6s\n", (int)indent, "", title, "TRY" ,"JIT", "STA", "COL");
+    fprintf(ostream,  "%*s%10s %6u %6u %6u %6u\n", (int)indent, "", sml,
+      internal_statistic[precision][0/*SML*/].ntry,
+      internal_statistic[precision][0/*SML*/].njit,
+      internal_statistic[precision][0/*SML*/].nsta,
+      internal_statistic[precision][0/*SML*/].ncol);
+    fprintf(ostream,  "%*s%10s %6u %6u %6u %6u\n", (int)indent, "", med,
+      internal_statistic[precision][1/*MED*/].ntry,
+      internal_statistic[precision][1/*MED*/].njit,
+      internal_statistic[precision][1/*MED*/].nsta,
+      internal_statistic[precision][1/*MED*/].ncol);
+    fprintf(ostream,  "%*s%10s %6u %6u %6u %6u\n", (int)indent, "", big,
+      internal_statistic[precision][2/*BIG*/].ntry,
+      internal_statistic[precision][2/*BIG*/].njit,
+      internal_statistic[precision][2/*BIG*/].nsta,
+      internal_statistic[precision][2/*BIG*/].ncol);
+    printed = 1;
+  }
+
+  return printed;
+}
+
+
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_register_static_code(const libxsmm_gemm_descriptor* desc,
+  unsigned int index, unsigned int hash, libxsmm_xmmfunction src, internal_regentry* registry)
 {
   internal_regkey* dst_key = internal_registry_keys + index;
   internal_regentry* dst_entry = registry + index;
-  assert(0 != desc && 0 != src.dmm && 0 != dst_key && 0 != registry && 0 != registered && 0 != total);
+  assert(0 != desc && 0 != src.dmm && 0 != dst_key && 0 != registry);
 
   if (0 != dst_entry->function.pmm) { /* collision? */
     /* start at a re-hashed index position */
@@ -428,19 +587,75 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_register_static_code(
 
     /* corresponding key position */
     dst_key = internal_registry_keys + i;
+
+    internal_update_statistic(desc, 0, 1);
   }
 
   if (0 == dst_entry->function.pmm) { /* registry not (yet) exhausted */
     dst_entry->function.xmm = src;
     dst_entry->size = 0; /* statically generated code */
     dst_key->descriptor = *desc;
-    ++(*registered);
   }
 
-  ++(*total);
+  internal_update_statistic(desc, 1, 0);
 }
 
 
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE int internal_get_prefetch(const libxsmm_gemm_descriptor* desc)
+{
+  assert(0 != desc);
+  switch (desc->prefetch) {
+    case LIBXSMM_PREFETCH_SIGONLY:            return 2;
+    case LIBXSMM_PREFETCH_BL2_VIA_C:          return 3;
+    case LIBXSMM_PREFETCH_AL2:                return 4;
+    case LIBXSMM_PREFETCH_AL2_AHEAD:          return 5;
+    case LIBXSMM_PREFETCH_AL2BL2_VIA_C:       return 6;
+    case LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD: return 7;
+    case LIBXSMM_PREFETCH_AL2_JPST:           return 8;
+    case LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST:  return 9;
+    default: {
+      assert(LIBXSMM_PREFETCH_NONE == desc->prefetch);
+      return 0;
+    }
+  }
+}
+
+
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_get_code_name(const char* target_arch,
+  const libxsmm_gemm_descriptor* desc, unsigned int buffer_size, char* name)
+{
+  assert((0 != desc && 0 != name) || 0 == buffer_size);
+  LIBXSMM_SNPRINTF(name, buffer_size, "libxsmm_%s_%c%c%c_%ux%ux%u_%u_%u_%u_a%i_b%i_p%i.jit",
+    target_arch /* code path name */,
+    0 == (LIBXSMM_GEMM_FLAG_F32PREC & desc->flags) ? 'd' : 's',
+    0 == (LIBXSMM_GEMM_FLAG_TRANS_A & desc->flags) ? 'n' : 't',
+    0 == (LIBXSMM_GEMM_FLAG_TRANS_B & desc->flags) ? 'n' : 't',
+    (unsigned int)desc->m, (unsigned int)desc->n, (unsigned int)desc->k,
+    (unsigned int)desc->lda, (unsigned int)desc->ldb, (unsigned int)desc->ldc,
+    desc->alpha, desc->beta, internal_get_prefetch(desc));
+}
+
+
+#if defined(LIBXSMM_VTUNE)
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_get_vtune_jitdesc(const internal_regentry* code, const char* name, LIBXSMM_VTUNE_JIT_DESC_TYPE* desc)
+{
+  assert(0 != code && 0 != code->id && 0 != code->size && 0 != desc);
+  desc->method_id = code->id;
+  /* incorrect constness (method_name) */
+  desc->method_name = (char*)name;
+  desc->method_load_address = code->function.pmm;
+  desc->method_size = code->size;
+  desc->line_number_size = 0;
+  desc->line_number_table = NULL;
+  desc->class_file_name = NULL;
+  desc->source_file_name = NULL;
+# if (2 == LIBXSMM_VTUNE_JITVERSION)
+  desc->module_name = "libxsmm.jit";
+# endif
+}
+#endif
+
+
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_regentry* internal_init(void)
 {
   /*const*/internal_regentry* result;
@@ -467,55 +682,13 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_regentry* internal_init(void)
 #endif
     if (0 == result) {
       int init_code;
-      const char *const env_jit = getenv("LIBXSMM_JIT");
-      if (env_jit && *env_jit) {
-        const int jit = atoi(env_jit);
-        if (0 == strcmp("0", env_jit)) { /* suppress running libxsmm_cpuid_x86 */
-          internal_target_archid = "generic";
-        }
-        else if (1 < jit) { /* suppress libxsmm_cpuid_x86 and override archid */
-          switch (LIBXSMM_X86_GENERIC + jit) {
-            case LIBXSMM_X86_AVX512: {
-              internal_target_arch = LIBXSMM_X86_AVX512;
-              internal_target_archid = "knl"; /* "skx" is fine too */
-            } break;
-            case LIBXSMM_X86_AVX2: {
-              internal_target_arch = LIBXSMM_X86_AVX2;
-              internal_target_archid = "hsw";
-            } break;
-            case LIBXSMM_X86_AVX: {
-              internal_target_arch = LIBXSMM_X86_AVX;
-              internal_target_archid = "snb";
-            } break;
-            default: if (LIBXSMM_X86_SSE3 <= (LIBXSMM_X86_GENERIC + jit)) {
-              internal_target_arch = LIBXSMM_X86_GENERIC + jit;
-              internal_target_archid = "sse";
-            }
-          }
-        }
-        else if (0 == strcmp("knl", env_jit) || 0 == strcmp("skx", env_jit)) {
-          internal_target_arch = LIBXSMM_X86_AVX512;
-          internal_target_archid = env_jit;
-        }
-        else if (0 == strcmp("hsw", env_jit)) {
-          internal_target_arch = LIBXSMM_X86_AVX2;
-          internal_target_archid = env_jit;
-        }
-        else if (0 == strcmp("snb", env_jit)) {
-          internal_target_arch = LIBXSMM_X86_AVX;
-          internal_target_archid = env_jit;
-        }
-      }
-      if (0 == internal_target_archid) {
-        internal_target_arch = libxsmm_cpuid_x86(&internal_target_archid);
-        assert(0 != internal_target_archid);
-      }
+      /* set internal_target_archid */
+      libxsmm_set_target_arch(getenv("LIBXSMM_TARGET"));
       { /* select prefetch strategy for JIT */
         const char *const env_prefetch = getenv("LIBXSMM_PREFETCH");
         if (0 == env_prefetch || 0 == *env_prefetch) {
           if (0 > LIBXSMM_PREFETCH) { /* permitted by LIBXSMM_PREFETCH_AUTO */
-            assert(0 != internal_target_archid);
-            internal_prefetch = 0 != strcmp("knl", internal_target_archid)
+            internal_prefetch = LIBXSMM_X86_AVX512_MIC != internal_target_archid
               ? LIBXSMM_PREFETCH_NONE : LIBXSMM_PREFETCH_AL2BL2_VIA_C;
           }
         }
@@ -533,8 +706,8 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_regentry* internal_init(void)
           }
         }
       }
-      libxsmm_hash_init(internal_target_arch);
-      libxsmm_gemm_diff_init(internal_target_arch);
+      libxsmm_hash_init(internal_target_archid);
+      libxsmm_gemm_diff_init(internal_target_archid);
       init_code = libxsmm_gemm_init(internal_target_archid, internal_prefetch);
 #if defined(__TRACE)
       const char *const env_trace_init = getenv("LIBXSMM_TRACE");
@@ -559,29 +732,22 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_regentry* internal_init(void)
         assert(0 == internal_registry_keys && 0 == internal_registry/*should never happen*/);
         result = (internal_regentry*)malloc(LIBXSMM_REGSIZE * sizeof(internal_regentry));
         internal_registry_keys = (internal_regkey*)malloc(LIBXSMM_REGSIZE * sizeof(internal_regkey));
-
         if (result && internal_registry_keys) {
+          const char *const env_verbose = getenv("LIBXSMM_VERBOSE");
+          internal_statistic_mnk = (unsigned int)(pow((double)(LIBXSMM_MAX_MNK), 0.3333333333333333) + 0.5);
+          if (0 != env_verbose && 0 != *env_verbose) {
+            internal_verbose = atoi(env_verbose);
+          }
           for (i = 0; i < LIBXSMM_REGSIZE; ++i) result[i].function.pmm = 0;
           /* omit registering code if JIT is enabled and if an ISA extension is found
            * which is beyond the static code path used to compile the library
            */
 #if (0 != LIBXSMM_JIT) && !defined(__MIC__)
-          if (LIBXSMM_STATIC_TARGET_ARCH >= internal_target_arch)
+          if (LIBXSMM_STATIC_TARGET_ARCH <= internal_target_archid && LIBXSMM_X86_AVX > internal_target_archid)
 #endif
           { /* opening a scope for eventually declaring variables */
-            unsigned int csp_tot = 0, csp_reg = 0, cdp_tot = 0, cdp_reg = 0;
             /* setup the dispatch table for the statically generated code */
 #           include <libxsmm_dispatch.h>
-#if !defined(NDEBUG) /* library code is expected to be mute */
-            if (csp_reg < csp_tot) {
-              fprintf(stderr, "LIBXSMM: %u of %u SP-kernels are not registered due to hash key collisions!\n", csp_tot - csp_reg, csp_tot);
-            }
-            if (cdp_reg < cdp_tot) {
-              fprintf(stderr, "LIBXSMM: %u of %u DP-kernels are not registered due to hash key collisions!\n", cdp_tot - cdp_reg, cdp_tot);
-            }
-#else
-            LIBXSMM_UNUSED(csp_tot); LIBXSMM_UNUSED(csp_reg); LIBXSMM_UNUSED(cdp_tot); LIBXSMM_UNUSED(cdp_reg);
-#endif
           }
           atexit(libxsmm_finalize);
 #if (defined(_REENTRANT) || defined(LIBXSMM_OPENMP)) && defined(LIBXSMM_GCCATOMICS)
@@ -677,7 +843,8 @@ LIBXSMM_RETARGETABLE void libxsmm_finalize(void)
       registry = internal_registry;
 
       if (0 != registry) {
-        void *const registry_keys = internal_registry_keys;
+        internal_regkey *const registry_keys = internal_registry_keys;
+        const char *const target_arch = internal_get_target_arch(internal_target_archid);
         /* serves as an id to invalidate the thread-local cache; never decremented */
         ++internal_teardown;
 #if defined(__TRACE)
@@ -706,46 +873,67 @@ LIBXSMM_RETARGETABLE void libxsmm_finalize(void)
         internal_registry = 0;
 #endif
         internal_registry_keys = 0;
-        { /* open scope to allocate variables */
-          LIBXSMM_DEBUG(unsigned int njit = 0, nstatic = 0;)
-          for (i = 0; i < LIBXSMM_REGSIZE; ++i) {
-            internal_regentry code = registry[i];
-            if (0 != code.function.pmm/*potentially allocated*/) {
-              if (0 != code.size/*JIT: actually allocated*/) {
-                /* make address valid by clearing an eventual collision flag */
-                code.function.imm &= ~LIBXSMM_HASH_COLLISION;
+        for (i = 0; i < LIBXSMM_REGSIZE; ++i) {
+          internal_regentry code = registry[i];
+          if (0 != code.function.pmm/*potentially allocated*/) {
+            const libxsmm_gemm_descriptor *const desc = &registry_keys[i].descriptor;
+            const unsigned long long size = LIBXSMM_MNK_SIZE(desc->m, desc->n, desc->k);
+            const int precision = (0 == (LIBXSMM_GEMM_FLAG_F32PREC & desc->flags) ? 0 : 1);
+            int bucket = 2;
+            if (LIBXSMM_MNK_SIZE(internal_statistic_sml, internal_statistic_sml, internal_statistic_sml) >= size) {
+              bucket = 0;
+            }
+            else if (LIBXSMM_MNK_SIZE(internal_statistic_med, internal_statistic_med, internal_statistic_med) >= size) {
+              bucket = 1;
+            }
+            if (0 != code.size/*JIT: actually allocated*/) {
+              /* make address valid by clearing an eventual collision flag */
+              code.function.imm &= ~LIBXSMM_HASH_COLLISION;
+#if defined(LIBXSMM_VTUNE)
+              if (0 != code.id && iJIT_SAMPLING_ON == iJIT_IsProfilingActive()) {
+                char jit_code_name[256];
+                LIBXSMM_VTUNE_JIT_DESC_TYPE vtune_jit_desc;
+                internal_get_code_name(target_arch, desc,
+                  sizeof(jit_code_name), jit_code_name);
+                internal_get_vtune_jitdesc(&code, jit_code_name, &vtune_jit_desc);
+                iJIT_NotifyEvent(LIBXSMM_VTUNE_JIT_UNLOAD, &vtune_jit_desc);
+              }
+#endif
 #if defined(_WIN32)
-                /* TODO: executable memory buffer under Windows */
+              /* TODO: executable memory buffer under Windows */
 #else
 # if defined(NDEBUG)
-                munmap(code.function.pmm, code.size);
+              munmap(code.function.pmm, code.size);
 # else /* library code is expected to be mute */
-                if (0 != munmap(code.function.pmm, code.size)) {
-                  const int error = errno;
-                  fprintf(stderr, "LIBXSMM: %s (munmap error #%i at %p+%u)!\n",
-                    strerror(error), error, code.function.pmm, code.size);
-                }
+              if (0 != munmap(code.function.pmm, code.size)) {
+                const int error = errno;
+                fprintf(stderr, "LIBXSMM: %s (munmap error #%i at %p+%u)!\n",
+                  strerror(error), error, code.function.pmm, code.size);
+              }
 # endif
 #endif
-                LIBXSMM_DEBUG(++njit;)
-              }
-              else {
-                LIBXSMM_DEBUG(++nstatic;)
-              }
+              ++internal_statistic[precision][bucket].njit;
+            }
+            else {
+              ++internal_statistic[precision][bucket].nsta;
             }
           }
-#if !defined(NDEBUG) /* library code is expected to be mute */
-          fprintf(stderr, "LIBXSMM_JIT=%s NJIT=%u NSTATIC=%u", 0 != internal_target_archid ? internal_target_archid : "0", njit, nstatic);
-          if (0 != internal_ncollisions) {
-            fprintf(stderr, ": %u hash key collisions handled!\n", internal_ncollisions);
-          }
-          else {
-            fprintf(stderr, "\n");
-          }
-#endif
         }
+        if (0 != internal_verbose) { /* print statistic on termination */
+          LIBXSMM_FLOCK(stderr);
+          LIBXSMM_FLOCK(stdout);
+          fflush(stdout); /* synchronize with standard output */
+          {
+            const unsigned int linebreak = 0 == internal_print_statistic(stderr, target_arch, 1/*SP*/, 1, 0) ? 1 : 0;
+            if (0 == internal_print_statistic(stderr, target_arch, 0/*DP*/, linebreak, 0) && 0 != linebreak) {
+              fprintf(stderr, "LIBXSMM_TARGET=%s\n", target_arch);
+            }
+          }
+          LIBXSMM_FUNLOCK(stdout);
+          LIBXSMM_FUNLOCK(stderr);
+        }
+        free((void*)registry_keys);
         free((void*)registry);
-        free(registry_keys);
       }
     }
 #if !defined(LIBXSMM_OPENMP) /* release locks */
@@ -755,30 +943,125 @@ LIBXSMM_RETARGETABLE void libxsmm_finalize(void)
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int libxsmm_get_target_arch()
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int libxsmm_get_target_archid(void)
 {
+  LIBXSMM_INIT
 #if !defined(_WIN32) && !defined(__MIC__) && (!defined(__CYGWIN__) || !defined(NDEBUG)/*code-coverage with Cygwin; fails@runtime!*/)
-  return internal_target_arch;
+  return internal_target_archid;
 #else /* no JIT support */
-  return LIBXSMM_TARGET_ARCH_GENERIC;
+  return LIBXSMM_MIN(internal_target_archid, LIBXSMM_X86_SSE4_2);
 #endif
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE const char* libxsmm_get_target_archid()
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_set_target_archid(int id)
 {
-  return internal_target_archid;
+  switch (id) {
+    case LIBXSMM_X86_AVX512_CORE:
+    case LIBXSMM_X86_AVX512_MIC:
+    case LIBXSMM_X86_AVX2:
+    case LIBXSMM_X86_AVX:
+    case LIBXSMM_X86_SSE4_2:
+    case LIBXSMM_X86_SSE4_1:
+    case LIBXSMM_X86_SSE3:
+    case LIBXSMM_TARGET_ARCH_GENERIC: {
+      internal_target_archid = id;
+    } break;
+    default: if (LIBXSMM_X86_GENERIC <= id) {
+      internal_target_archid = LIBXSMM_X86_GENERIC;
+    }
+    else {
+      internal_target_archid = LIBXSMM_TARGET_ARCH_UNKNOWN;
+    }
+  }
+
+#if !defined(NDEBUG) /* library code is expected to be mute */
+  {
+    const int cpuid = libxsmm_cpuid_x86();
+    if (cpuid < internal_target_archid) {
+      fprintf(stderr, "LIBXSMM: \"%s\" code will fail to run on \"%s\"!\n",
+        internal_get_target_arch(internal_target_archid),
+        internal_get_target_arch(cpuid));
+    }
+  }
+#endif
+}
+
+
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE const char* libxsmm_get_target_arch(void)
+{
+  LIBXSMM_INIT
+  return internal_get_target_arch(internal_target_archid);
 }
 
 
 /* function serves as a helper for implementing the Fortran interface */
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void get_target_archid(char* name, int length)
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void get_target_arch(char* arch, int length);
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void get_target_arch(char* arch, int length)
 {
-  const char* c = internal_target_archid ? internal_target_archid : "";
+  const char* c = libxsmm_get_target_arch();
   int i;
-  assert(0 != name); /* valid here since function is not in the public interface */
-  for (i = 0; i < length && 0 != *c; ++i, ++c) name[i] = *c;
-  for (; i < length; ++i) name[i] = ' ';
+  assert(0 != arch); /* valid here since function is not in the public interface */
+  for (i = 0; i < length && 0 != *c; ++i, ++c) arch[i] = *c;
+  for (; i < length; ++i) arch[i] = ' ';
+}
+
+
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_set_target_arch(const char* arch)
+{
+  int target_archid = LIBXSMM_TARGET_ARCH_UNKNOWN;
+
+  if (arch && *arch) {
+    const int jit = atoi(arch);
+    if (0 == strcmp("0", arch)) {
+      target_archid = LIBXSMM_TARGET_ARCH_GENERIC;
+    }
+    else if (1 < jit) {
+      target_archid = LIBXSMM_X86_GENERIC + jit;
+    }
+    else if (0 == strcmp("skx", arch) || 0 == strcmp("avx3", arch) || 0 == strcmp("avx512", arch)) {
+      target_archid = LIBXSMM_X86_AVX512_CORE;
+    }
+    else if (0 == strcmp("knl", arch) || 0 == strcmp("mic2", arch)) {
+      target_archid = LIBXSMM_X86_AVX512_MIC;
+    }
+    else if (0 == strcmp("hsw", arch) || 0 == strcmp("avx2", arch)) {
+      target_archid = LIBXSMM_X86_AVX2;
+    }
+    else if (0 == strcmp("snb", arch) || 0 == strcmp("avx", arch)) {
+      target_archid = LIBXSMM_X86_AVX;
+    }
+    else if (0 == strcmp("wsm", arch) || 0 == strcmp("nhm", arch) || 0 == strcmp("sse4", arch) || 0 == strcmp("sse4_2", arch) || 0 == strcmp("sse4.2", arch)) {
+      target_archid = LIBXSMM_X86_SSE4_2;
+    }
+    else if (0 == strcmp("sse4_1", arch) || 0 == strcmp("sse4.1", arch)) {
+      target_archid = LIBXSMM_X86_SSE4_1;
+    }
+    else if (0 == strcmp("sse3", arch) || 0 == strcmp("sse", arch)) {
+      target_archid = LIBXSMM_X86_SSE3;
+    }
+    else if (0 == strcmp("x86", arch) || 0 == strcmp("sse2", arch)) {
+      target_archid = LIBXSMM_X86_GENERIC;
+    }
+    else if (0 == strcmp("generic", arch) || 0 == strcmp("none", arch)) {
+      target_archid = LIBXSMM_TARGET_ARCH_GENERIC;
+    }
+  }
+
+  if (LIBXSMM_TARGET_ARCH_UNKNOWN == target_archid || LIBXSMM_X86_AVX512_CORE < target_archid) {
+    target_archid = libxsmm_cpuid_x86();
+  }
+#if !defined(NDEBUG) /* library code is expected to be mute */
+  else {
+    const int cpuid = libxsmm_cpuid_x86();
+    if (cpuid < target_archid) {
+      fprintf(stderr, "LIBXSMM: \"%s\" code will fail to run on \"%s\"!\n",
+        internal_get_target_arch(target_archid),
+        internal_get_target_arch(cpuid));
+    }
+  }
+#endif
+  internal_target_archid = target_archid;
 }
 
 
@@ -786,6 +1069,7 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
 {
 #if (0 != LIBXSMM_JIT)
 # if !defined(_WIN32) && !defined(__MIC__) && (!defined(__CYGWIN__) || !defined(NDEBUG)/*code-coverage with Cygwin; fails@runtime!*/)
+  const char *const target_arch = internal_get_target_arch(internal_target_archid);
   libxsmm_generated_code generated_code;
   assert(0 != desc && 0 != code);
   assert(0 != internal_target_archid);
@@ -799,7 +1083,7 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
   generated_code.last_error = 0;
 
   /* generate kernel */
-  libxsmm_generator_gemm_kernel(&generated_code, desc, internal_target_archid);
+  libxsmm_generator_gemm_kernel(&generated_code, desc, target_arch);
 
   /* handle an eventual error in the else-branch */
   if (0 == generated_code.last_error) {
@@ -848,27 +1132,34 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
         memcpy(code->function.pmm, generated_code.generated_code, generated_code.code_size);
 
         if (0/*ok*/ == mprotect(code->function.pmm, generated_code.code_size, PROT_EXEC | PROT_READ)) {
-# if !defined(NDEBUG) && defined(_DEBUG)
-          /* write buffer for manual decode as binary to a file */
-          char objdump_name[512];
-          FILE* byte_code;
-          sprintf(objdump_name, "kernel_%s_f%i_%c%c_m%u_n%u_k%u_lda%u_ldb%u_ldc%u_a%i_b%i_pf%i.bin",
-            internal_target_archid /* best available/supported code path */,
-            0 == (LIBXSMM_GEMM_FLAG_F32PREC & desc->flags) ? 64 : 32,
-            0 == (LIBXSMM_GEMM_FLAG_TRANS_A & desc->flags) ? 'n' : 't',
-            0 == (LIBXSMM_GEMM_FLAG_TRANS_B & desc->flags) ? 'n' : 't',
-            desc->m, desc->n, desc->k, desc->lda, desc->ldb, desc->ldc,
-            desc->alpha, desc->beta, desc->prefetch);
-          byte_code = fopen(objdump_name, "wb");
-          if (0 != byte_code) {
-            fwrite(generated_code.generated_code, 1, generated_code.code_size, byte_code);
-            fclose(byte_code);
-          }
-# endif /*!defined(NDEBUG) && defined(_DEBUG)*/
-          /* free temporary/initial code buffer */
-          free(generated_code.generated_code);
+# if (!defined(NDEBUG) && defined(_DEBUG)) || defined(LIBXSMM_VTUNE)
+          char jit_code_name[256];
+          internal_get_code_name(target_arch, desc, sizeof(jit_code_name), jit_code_name);
+# endif
           /* finalize code generation */
           code->size = generated_code.code_size;
+          /* free temporary/initial code buffer */
+          free(generated_code.generated_code);
+# if !defined(NDEBUG) && defined(_DEBUG)
+          { /* dump byte-code into file */
+            FILE *const byte_code = fopen(jit_code_name, "wb");
+            if (0 != byte_code) {
+              fwrite(code->function.pmm, 1, code->size, byte_code);
+              fclose(byte_code);
+            }
+          }
+# endif /*!defined(NDEBUG) && defined(_DEBUG)*/
+# if defined(LIBXSMM_VTUNE)
+          if (iJIT_SAMPLING_ON == iJIT_IsProfilingActive()) {
+            LIBXSMM_VTUNE_JIT_DESC_TYPE vtune_jit_desc;
+            code->id = iJIT_GetNewMethodID();
+            internal_get_vtune_jitdesc(code, jit_code_name, &vtune_jit_desc);
+            iJIT_NotifyEvent(LIBXSMM_VTUNE_JIT_LOAD, &vtune_jit_desc);
+          }
+          else {
+            code->id = 0;
+          }
+# endif
         }
         else { /* there was an error with mprotect */
 # if defined(NDEBUG)
@@ -938,7 +1229,7 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
 #   endif
   LIBXSMM_UNUSED(desc); LIBXSMM_UNUSED(code);
   /* libxsmm_get_target_arch also serves as a runtime check whether JIT is available or not */
-  assert(LIBXSMM_X86_AVX > libxsmm_get_target_arch());
+  assert(LIBXSMM_X86_AVX > internal_target_archid);
 # endif /*_WIN32*/
 #endif /*LIBXSMM_JIT*/
 }
@@ -978,3 +1269,31 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE libxsmm_dmmfunction libxsmm_dmmdispatch(in
   INTERNAL_DMMDISPATCH(flags, m, n, k, lda, ldb, ldc, alpha, beta, prefetch);
 }
 
+
+#if defined(LIBXSMM_GEMM_EXTWRAP)
+#if defined(__STATIC)
+
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void LIBXSMM_FSYMBOL(__real_sgemm)(
+  const char* transa, const char* transb,
+  const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
+  const float* alpha, const float* a, const libxsmm_blasint* lda,
+  const float* b, const libxsmm_blasint* ldb,
+  const float* beta, float* c, const libxsmm_blasint* ldc)
+{
+  sgemm_(transa, transb, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
+}
+
+
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void LIBXSMM_FSYMBOL(__real_dgemm)(
+  const char* transa, const char* transb,
+  const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
+  const double* alpha, const double* a, const libxsmm_blasint* lda,
+  const double* b, const libxsmm_blasint* ldb,
+  const double* beta, double* c, const libxsmm_blasint* ldc)
+{
+  dgemm_(transa, transb, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
+}
+
+#endif /*defined(__STATIC)*/
+#endif /*defined(LIBXSMM_GEMM_EXTWRAP)*/
+
