@@ -34,44 +34,34 @@
 #include "libxsmm_gemm.h"
 
 #if !defined(LIBXSMM_GEMM_EXTWRAP) && defined(__GNUC__) && !defined(_WIN32) && !(defined(__APPLE__) && defined(__MACH__) && \
-  LIBXSMM_VERSION3(6, 1, 0) >= LIBXSMM_VERSION3(__clang_major__, __clang_minor__, __clang_patchlevel__)) && !defined(__CYGWIN__)
+  LIBXSMM_VERSION3(6, 1, 0) >= LIBXSMM_VERSION3(__clang_major__, __clang_minor__, __clang_patchlevel__))
 # if defined(__STATIC) /* -Wl,--wrap=xgemm_ */
 #   define LIBXSMM_GEMM_EXTWRAP
 #   define LIBXSMM_GEMM_WEAK_SLIB LIBXSMM_ATTRIBUTE(weak)
 #   define LIBXSMM_GEMM_EXTWRAP_SGEMM LIBXSMM_FSYMBOL(__wrap_sgemm)
 #   define LIBXSMM_GEMM_EXTWRAP_DGEMM LIBXSMM_FSYMBOL(__wrap_dgemm)
-    LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(weak) void LIBXSMM_FSYMBOL(__real_sgemm)(
+    LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void LIBXSMM_FSYMBOL(__real_sgemm)(
       const char*, const char*, const libxsmm_blasint*, const libxsmm_blasint*, const libxsmm_blasint*,
       const float*, const float*, const libxsmm_blasint*, const float* b, const libxsmm_blasint*,
-      const float* beta, float*, const libxsmm_blasint*);
-    LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(weak) void LIBXSMM_FSYMBOL(__real_dgemm)(
+      const float*, float*, const libxsmm_blasint*);
+    LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void LIBXSMM_FSYMBOL(__real_dgemm)(
       const char*, const char*, const libxsmm_blasint*, const libxsmm_blasint*, const libxsmm_blasint*,
       const double*, const double*, const libxsmm_blasint*, const double* b, const libxsmm_blasint*,
-      const double* beta, double*, const libxsmm_blasint*);
-    LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(weak) void LIBXSMM_FSYMBOL(__real_mkl_sgemm)(
-      const char*, const char*, const libxsmm_blasint*, const libxsmm_blasint*, const libxsmm_blasint*,
-      const float*, const float*, const libxsmm_blasint*, const float* b, const libxsmm_blasint*,
-      const float* beta, float*, const libxsmm_blasint*);
-    LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(weak) void LIBXSMM_FSYMBOL(__real_mkl_dgemm)(
-      const char*, const char*, const libxsmm_blasint*, const libxsmm_blasint*, const libxsmm_blasint*,
-      const double*, const double*, const libxsmm_blasint*, const double* b, const libxsmm_blasint*,
-      const double* beta, double*, const libxsmm_blasint*);
+      const double*, double*, const libxsmm_blasint*);
 # elif !defined(__CYGWIN__) /* LD_PRELOAD */
 #   define LIBXSMM_GEMM_EXTWRAP
 #   define LIBXSMM_GEMM_WEAK_DLIB LIBXSMM_ATTRIBUTE(weak)
 #   define LIBXSMM_GEMM_EXTWRAP_SGEMM LIBXSMM_FSYMBOL(sgemm)
 #   define LIBXSMM_GEMM_EXTWRAP_DGEMM LIBXSMM_FSYMBOL(dgemm)
 # endif
-  /* avoid remark about external function definition with no prior declaration */
   LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void LIBXSMM_GEMM_EXTWRAP_SGEMM(
     const char*, const char*, const libxsmm_blasint*, const libxsmm_blasint*, const libxsmm_blasint*,
     const float*, const float*, const libxsmm_blasint*, const float* b, const libxsmm_blasint*,
-    const float* beta, float*, const libxsmm_blasint*);
-  /* avoid remark about external function definition with no prior declaration */
+    const float*, float*, const libxsmm_blasint*);
   LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void LIBXSMM_GEMM_EXTWRAP_DGEMM(
     const char*, const char*, const libxsmm_blasint*, const libxsmm_blasint*, const libxsmm_blasint*,
     const double*, const double*, const libxsmm_blasint*, const double* b, const libxsmm_blasint*,
-    const double* beta, double*, const libxsmm_blasint*);
+    const double*, double*, const libxsmm_blasint*);
 #endif /*defined(LIBXSMM_GEMM_EXTWRAP)*/
 
 #if !defined(LIBXSMM_GEMM_WEAK_SLIB)
@@ -82,6 +72,16 @@
 #endif
 
 
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void LIBXSMM_FSYMBOL(sgemm)(
+  const char*, const char*, const libxsmm_blasint*, const libxsmm_blasint*, const libxsmm_blasint*,
+  const float*, const float*, const libxsmm_blasint*, const float*, const libxsmm_blasint*,
+  const float*, float*, const libxsmm_blasint*);
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void LIBXSMM_FSYMBOL(dgemm)(
+  const char*, const char*, const libxsmm_blasint*, const libxsmm_blasint*, const libxsmm_blasint*,
+  const double*, const double*, const libxsmm_blasint*, const double*, const libxsmm_blasint*,
+  const double*, double*, const libxsmm_blasint*);
+
+
 /** INTERNAL: configuration table containing the tile sizes separate for DP and SP. */
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int libxsmm_internal_tile_size[/*DP/SP*/][3/*TILE_M,TILE_N,TILE_K*/];
 /** INTERNAL: number of threads per core */
