@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2009-2015, Intel Corporation                                **
+** Copyright (c) 2009-2016, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -30,7 +30,7 @@
 ******************************************************************************/
 #include <libxsmm_timer.h>
 
-#if defined(LIBXSMM_OFFLOAD_BUILD)
+#if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
 #if defined(_WIN32)
@@ -39,7 +39,7 @@
 # include <sys/time.h>
 # include <time.h>
 #endif
-#if defined(LIBXSMM_OFFLOAD_BUILD)
+#if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
 #endif
 
